@@ -4,16 +4,7 @@ kaczmarz <- function(fl,R,eps=getOption('lfe.eps'),init=NULL,threads=getOption('
   if(is.null(threads)) threads <- 1
   islist <- is.list(R)
   if(!islist) R <- list(R)
-  # Now, each element of the list is either
-  # a matrix or a vector, let's reorder randomly
-  oo <- order(runif(length(fl[[1]])))
-  ofl <- lapply(fl,function(f) f[oo])
-  oR <- lapply(R,function(l) if(is.array(l)) apply(l,2,function(f) f[oo]) else l[oo])
-
-#  v <- .Call('kaczmarz',fl,R,eps,as.vector(init),as.integer(threads),PACKAGE='lfe')
-
-  v <- .Call('kaczmarz',ofl,oR,eps,as.vector(init),as.integer(threads),PACKAGE='lfe')
-  
+  v <- .Call('kaczmarz',fl,R,eps,as.vector(init),as.integer(threads),PACKAGE='lfe')
   if(!islist) {
     v <- drop(v[[1]])
   }
@@ -23,12 +14,15 @@ kaczmarz <- function(fl,R,eps=getOption('lfe.eps'),init=NULL,threads=getOption('
 getfe.kaczmarz <- function(obj,se=FALSE,eps=getOption('lfe.eps'),ef='ref',bN=100) {
 
   R <- obj$residuals-obj$full.residuals
+  if(isTRUE(getOption('lfe.randkac'))) {
 # order randomly
-#  oo <- order(runif(length(R)))
-#  Roo <- R[oo]
-#  fe <- lapply(obj$fe,function(f) f[oo])
-#  v <- kaczmarz(fe,Roo,eps)
-  v <- kaczmarz(obj$fe,R,eps)
+    oo <- order(runif(length(R)))
+    Roo <- R[oo]
+    fe <- lapply(obj$fe,function(f) f[oo])
+    v <- kaczmarz(fe,Roo,eps)
+  } else {
+    v <- kaczmarz(obj$fe,R,eps)
+  }
   nm <- names(v)
   if(is.character(ef)) {
     ef <- efactory(obj,opt=ef)
@@ -281,9 +275,7 @@ btrap <- function(alpha,obj,N=100,ef=NULL,eps=getOption('lfe.eps'),threads=getOp
   R <- obj$residuals-obj$full.residuals
   j <- 0 #avoid warning from check
 
-  # My intuition is that it ought to be possible to avoid the repeated demeanlist:
-  # but how?
-  # now, we use the variance in PY-PXbeta to generate variation V (i.e. the full.residuals),
+  # We use the variance in PY-PXbeta to generate variation V (i.e. the full.residuals),
   # this is the outcome residuals, so the correct way to do it.
   # Then we compute W=(I-P)V.  Now, W is a vector which has constant
   # values for each level of each factor.  Couldn't we generate it by using a level
@@ -332,11 +324,25 @@ btrap <- function(alpha,obj,N=100,ef=NULL,eps=getOption('lfe.eps'),threads=getOp
   Rdup <- matrix(rep(R,vpb),length(smpdraw),vpb)
   gc()
 
+  if(isTRUE(getOption('lfe.randkac'))) {
+    # Now, each element of the list is either
+    # a matrix or a vector, let's reorder randomly
+    oo <- order(runif(length(obj$fe[[1]])))
+    ofl <- lapply(obj$fe,function(f) f[oo])
+  }
+  
+
   for(i in 1:blks) {
     rsamp <- sample(smpdraw,vpb*length(smpdraw),replace=TRUE)
     dim(rsamp) <- c(length(smpdraw),vpb)
     newR <- rsamp - demeanlist(rsamp,obj$fe,eps=eps,threads=threads,quiet=TRUE) + Rdup
-    v <- kaczmarz(obj$fe,newR,eps,threads=threads)*sefact
+    if(isTRUE(getOption('lfe.randkac'))) {
+      oR <- apply(newR,2,function(l) l[oo])
+      v <- kaczmarz(ofl,oR,eps,threads=threads)*sefact
+      rm(oR)
+    } else {
+      v <- kaczmarz(obj$fe,newR,eps,threads=threads)*sefact
+    }
     efv <- apply(v,2,ef,addnames=FALSE)
     rm(v,rsamp,newR); gc()
     vsum <- vsum + rowSums(efv)
@@ -366,6 +372,11 @@ is.estimable <- function(ef,fe,R=NULL) {
     dim(vec) <- c(nr,nc)
     R <- rowSums(vec)
   }
+  if(isTRUE(getOption('lfe.randkac'))) {
+    oo <- order(runif(length(fe[[1]])))
+    fe <- lapply(fe,function(f) f[oo])
+    R <- R[oo]
+  }
   v1 <- ef(kaczmarz(fe,R,init=runif(N)),TRUE)
   v2 <- ef(kaczmarz(fe,R,init=runif(N)),TRUE)
   df <- sqrt(sum((v1-v2)**2))
